# -*- coding: utf-8 -*-

# Imports: Zope
from Acquisition    import aq_inner, aq_parent
from Products.Five  import BrowserView
from zope.interface import implements

# Imports: CMF & Plone
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import normalizeString
from Products.ATContentTypes.interface.topic import IATTopic

# Imports: Structured Document
from Products.StructuredDocument.security   import CanViewComments
from Products.StructuredDocument.interfaces import INextPreviousView
from Products.StructuredDocument.interfaces import INextPreviousProvider
from Products.StructuredDocument.interfaces import IDocumentContentProvider
from Products.StructuredDocument.interfaces import IChapterContentProvider
from Products.StructuredDocument.interfaces import IStructuredActionsProvider
from Products.StructuredDocument.interfaces import IAddableContentProvider
from Products.StructuredDocument.interfaces import IListingContentProvider
from Products.StructuredDocument.interfaces import ICommentsProvider


class NextPreviousView( BrowserView ):
    """ Next-Previous view """

    implements( INextPreviousView )

    def __init__(self, context, request):
        self.context = context
        self.request = request


    def next(self):
        provider = self.providedParent()
        if provider is not None:
            return provider.getNextItem(self.context.getId())
        return None
    

    def previous(self):
        provider = self.providedParent()
        if provider is not None:
            return provider.getPreviousItem(self.context.getId())
        return None


    def next_previous(self):
        provider = self.providedParent()
        if provider is not None:
            return provider.itemRelatives(self.context.getId())
        return None


    def providedParent(self):
        parent = aq_parent(aq_inner(self.context))
        return INextPreviousProvider(parent, None)


class ListingContentProvider( BrowserView ):
    """ Access contents of a folder """

    implements( IListingContentProvider )
    
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getContents(self):
        if IATTopic.providedBy(self.context):
            return self.context.queryCatalog()
        return self.context.getFolderContents()
    
    def normalizeString(self, text, context=None, encoding=None, relaxed=False):
        return normalizeString(text, context=self.context, encoding=encoding, relaxed=relaxed)

    def getViewUrl(self, brain):
        props = getToolByName(self.context, 'portal_properties')       
        view_action_types = props.site_properties.getProperty('typesUseViewActionInListings', ())
    
        if brain.portal_type in view_action_types:
            return brain.getURL() + '/view'
    
        return brain.getURL()


class AddableContentProvider( BrowserView ):
    """ Access addable contents """

    implements( IAddableContentProvider )
    
    def __init__(self, context, request):
        self.context = context
        self.request = request


    def addable_types(self):
        putils = getToolByName(self.context, 'plone_utils')

        allowed       = self.context.allowedContentTypes()
        notAddable    = self.context.getNotAddableTypes()
        sortedAllowed = self.context.sortObjects(allowed)

        addable_types = []
        for possibleType in sortedAllowed:
            type_id = possibleType.getId()
            if type_id not in notAddable:
                addable_type = dict(
                    name  = type_id,
                    title = self.context.translate(possibleType.Title()),
                    desc  = possibleType.Description(),
                    klass = 'contenttype-' + putils.normalizeString(type_id),
                    add_url = '%s/createObject?type_name=%s' % (self.context.absolute_url(), type_id) 
                    )
                addable_types.append(addable_type)
        return addable_types


    def can_add_content(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Add portal content', self.context)


class DocumentContentProvider( BrowserView ):
    """ Access document's contents """

    implements( IDocumentContentProvider )
    
    def __init__(self, context, request):
        self.context = context
        self.request = request


    def chapters(self):
        filter = dict(object_implements = 'Products.StructuredDocument.interfaces.chapter.IStructuredChapter')
        return self.context.getFolderContents(contentFilter = filter)


    def information_blocks(self):
        filter = dict(object_implements = 'Products.StructuredDocument.interfaces.blocks.IStructuredBlock')
        return self.context.getFolderContents(contentFilter = filter,
                                              full_objects  = True)

    def can_modify_content(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Modify portal content', self.context)

    def can_add_content(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Add portal content', self.context)


class ChapterContentProvider( BrowserView ):
    """ Access chapter's contents """

    implements( IChapterContentProvider )
    
    def __init__(self, context, request):
        self.context = context
        self.request = request


    def paragraphs(self):
        filter = dict(object_implements = 'Products.StructuredDocument.interfaces.paragraph.IStructuredParagraph')
        return self.context.getFolderContents(contentFilter = filter)


    def can_modify_content(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Modify portal content', self.context)


    def can_add_content(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Add portal content', self.context)


    def chapter_ordering(self):
        """ Order the chapter """
        
        oid      = self.request.get('id', None)
        url      = self.request['HTTP_REFERER']
        utils    = getToolByName(self.context, 'plone_utils')
        response = self.request.RESPONSE
        position = self.request.get('position', None)

        # We need both arguments
        if not position or not oid:
            msg = _(u"An error occured while sorting the chapter.")
            utils.addPortalMessage(msg)
            return response.redirect(url)

        # The actions provided to reorganize
        position = position.lower()
        actions  = dict(
            up      = 'moveObjectsUp',
            down    = 'moveObjectsDown',
            top     = 'moveObjectsToTop',
            bottom  = 'moveObjectsToBottom',
            ordered = 'orderObjects',
            )

        # Get the action or bust !
        if position in actions:
            action = getattr(self.context.aq_inner, actions[position], None)
            if not action:
                msg = _(u"An error occured while sorting the chapter.")
                utils.addPortalMessage(msg)
                return response.redirect(url)

        # Just do it !
        action(oid)
        utils.reindexOnReorder(self.context)

        # Everything went smoothly.
        # Now, get me back where I came from !
        msg = _(u'Item\'s position has changed.')
        utils.addPortalMessage(msg)
        return response.redirect("%s#%s" % (url, oid))


class StructuredActionsProvider( BrowserView ):
    """ Provides an easy way to get actions from any AT object """

    implements( IStructuredActionsProvider )
    
    def __init__(self, context, request):
        self.context = context
        self.request = request


    def language(self):
        props = getToolByName(self.context, 'portal_properties')
        site_props = props.site_properties
        default_language = site_props.getProperty('default_language', None)
        
        return self.request.get('language', None) or \
               self.context.Language() or default_language

        
    def actions(self):
        pa = getToolByName(self.context, 'portal_actions')
        return pa.listFilteredActionsFor(self.context)

        
    def view_actions(self, template_id):
        default_tab = self.request.get('default_tab', 'view')
        return self.context.plonifyActions(template_id=template_id,
                                           actions=self.actions(),
                                           default_tab=default_tab)

    def can_modify_content(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Modify portal content', self.context)


class CommentsProvider( BrowserView ):
    """ Provides  """

    implements( ICommentsProvider )
    
    def __init__(self, context, request):
        self.context = context


    def allowed_comments(self):
        portal_discussion = getToolByName(self.context, 'portal_discussion')
        return portal_discussion.isDiscussionAllowedFor(self.context)


    def delete_comments(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Manage portal', self.context)


    def can_view_comments(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission(CanViewComments, self.context)
    

    def add_comments(self):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.checkPermission('Reply to item', self.context)


    def comments(self):
        return self.context.getReplyReplies(self.context)


    def comment_creator(self, userid):
        mtool = getToolByName(self.context, 'portal_membership')
        return mtool.getMemberInfo(userid)


    def toLocalizedTime(self, time, long_format=None):
        tool = getToolByName(self.context, 'translation_service')
        return tool.ulocalized_time(time, long_format, self.context,
                                    domain='plone')


    def display_add_buttons(self):
        return self.allowed_comments() and self.add_comments()
